program FullScreenDialog_api;
{******************************************************************}
{                                                                  }
{       Borland Delphi DirectX 8 SDK Examples                      }
{       Conversion of the Full Screen Dialog Example               }
{                                                                  }
{ Portions created by Microsoft are                                }
{ Copyright (C) 1995-2000 Microsoft Corporation.                   }
{ All Rights Reserved.                                             }
{                                                                  }
{ The original files are : FullScreenDialog.h,                     }
{                          FullScreenDialog.cpp                    }
{ The original Pascal code is : FullScreenDialog_api.pas           }
{ The initial developer of the Pascal code is : Dominique Louis    }
{ ( Dominique@SavageSoftware.com.au )                              }
{                                                                  }
{ Portions created by Dominique Louis are                          }
{ Copyright (C) 2000 - 2001 Dominique Louis.                       }
{                                                                  }
{ Contributor(s)                                                   }
{ --------------                                                   }
{ <Contributer Name> ( contributer@sp.sw )                         }
{                                                                  }
{ Obtained through:                                                }
{ Joint Endeavour of Delphi Innovators ( Project JEDI )            }
{                                                                  }
{ You may retrieve the latest version of this file at the Project  }
{ JEDI home page, located at http://delphi-jedi.org                }
{                                                                  }
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/NPL/NPL-1_1Final.html                     }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{ Description                                                      }
{ -----------                                                      }
{   FullScreenDialog demonstrates how to display a GDI dialog      }
{   while using DirectDraw in full-screen exclusive mode.          }
{                                                                  }
{ Requires                                                         }
{ --------                                                         }
{   Eric Unger and Tim Baumgarten's conversions of the DirectX     }
{   headers. They are available from...                            }
{   http://delphi-jedi.org/DelphiGraphics/ .                       }
{                                                                  }
{                                                                  }
{ Programming Notes                                                }
{ -----------------                                                }
{   For details on how to setup a full-screen DirectDraw app, see  }
{   the FullScreenMode sample.                                     }
{                                                                  }
{   If the display device supports DDCAPS2_CANRENDERWINDOWED then  }
{   to make GDI write to a DirectDraw surface, call                }
{   IDirectDraw::FlipToGDISurface, then create a clipper object    }
{   which GDI uses when drawing.  To display a dialog, then simply }
{   create and show it as normal.                                  }
{                                                                  }
{   If the display device does not support                         }
{   DDCAPS2_CANRENDERWINDOWED (some secondary                      }
{   graphics cards are like this) then the card does not support   }
{   FlipToGDISurface.                                              }
{   So it is necessary instead to have GDI make a bitmap of the    }
{   dialog to be drawn, then blt this bitmap to the back buffer.   }
{   However, this sample does not show how to do this technique,   }
{                                                                  }
{ Revision History                                                 }
{ ----------------                                                 }
{   2000-12-01 - DL  Initial translation.                          }
{                                                                  }
{                                                                  }
{******************************************************************}

uses
  Windows,
  SysUtils,
  Messages,
  DirectDraw,
  DDUtil in '..\..\Common\DDUtil.pas';

{$R *.RES}
const
  NAME : PChar = 'FullScreenDialog';
  TITLE : PChar = 'DirectDraw Full Screen Dialog Example';
  HELPTEXT : PChar = 'Press Escape to quit.  Press F1 to bring up the dialog.';

  IDI_MAIN_ICON = 101;
  IDR_MAIN_ACCEL = 103;
  IDD_DIALOG_SAMPLE = 106;
  IDB_DIRECTX = 107;
  IDM_EXIT = 1001;
  IDC_RADIO1 = 1003;
  IDC_RADIO2 = 1004;
  IDC_RADIO3 = 1005;
  IDC_EDIT1 = 1006;
  IDC_COMBO1 = 1007;
  IDM_SHOW_DIALOG = 40001;

  SCREEN_WIDTH = 640;
  SCREEN_HEIGHT = 480;
  SCREEN_BPP = 8;

  SPRITE_DIAMETER = 48;
  NUM_SPRITES = 25;



type
  PSpriteRecord = ^TSpriteRecord;
  TSpriteRecord = packed record
    fPosX : single;
    fPosY : single;
    fVelX : single;
    fVelY : single;
  end;

var
  g_pDisplay : TDisplay = nil;
  g_pLogoSurface : TSurface = nil;
  g_pTextSurface : TSurface = nil;
  g_hWndDlg : HWND = 0;
  g_bActive : Boolean = False; // Is application active?
  g_dwLastTick : DWord;
  g_Sprite : array[ 0..NUM_SPRITES - 1 ] of TSpriteRecord;


{-----------------------------------------------------------------------------
  Name: FreeDirectDraw
  Desc: Release all the DirectDraw objects
-----------------------------------------------------------------------------}

procedure FreeDirectDraw;
begin
  g_pLogoSurface.Free;
  g_pLogoSurface := nil;
  g_pTextSurface.Free;
  g_pTextSurface := nil;
  g_pDisplay.Free;
  g_pDisplay := nil;
end;

{-----------------------------------------------------------------------------
   Name: UpdateSprite()
   Desc: Move the sprite around and make it bounce based on how much time
         has passed
-----------------------------------------------------------------------------}

procedure UpdateSprite( var pSprite : TSpriteRecord; fTimeDelta : Single );
begin
  // Update the sprite position
  pSprite.fPosX := pSprite.fPosX + pSprite.fVelX * fTimeDelta;
  pSprite.fPosY := pSprite.fPosY + pSprite.fVelY * fTimeDelta;

  // Clip the position, and bounce if it hits the edge
  if ( pSprite.fPosX < 0.0 ) then
  begin
    pSprite.fPosX := 0;
    pSprite.fVelX := -pSprite.fVelX;
  end;

  if ( pSprite.fPosX >= SCREEN_WIDTH - SPRITE_DIAMETER ) then
  begin
    pSprite.fPosX := SCREEN_WIDTH - 1 - SPRITE_DIAMETER;
    pSprite.fVelX := -pSprite.fVelX;
  end;

  if ( pSprite.fPosY < 0 ) then
  begin
    pSprite.fPosY := 0;
    pSprite.fVelY := -pSprite.fVelY;
  end;

  if ( pSprite.fPosY > SCREEN_HEIGHT - SPRITE_DIAMETER ) then
  begin
    pSprite.fPosY := SCREEN_HEIGHT - 1 - SPRITE_DIAMETER;
    pSprite.fVelY := -pSprite.fVelY;
  end;
end;

{-----------------------------------------------------------------------------
   Name: DisplayFrame()
   Desc: Blts a the sprites to the back buffer, then it blts or flips the
         back buffer onto the primary buffer.
-----------------------------------------------------------------------------}

function DisplayFrame : HRESULT;
var
  hr : HRESULT;
  iSprite : integer;
  pddsBackBuffer, pddsFrontBuffer : IDIRECTDRAWSURFACE7;
begin
  // Fill the back buffer with black, ignoring errors until the flip
  g_pDisplay.Clear( 0 );

  // Blt the help text on the backbuffer, ignoring errors until the flip
  g_pDisplay.Blt( 10, 10, g_pTextSurface, nil );

  // Blt all the sprites onto the back buffer using color keying,
  // ignoring errors until the flip. Note that all of these sprites
  // use the same DirectDraw surface.
  for iSprite := 0 to NUM_SPRITES - 1 do
    g_pDisplay.Blt( Round( g_Sprite[ iSprite ].fPosX ), Round( g_Sprite[ iSprite ].fPosY ), g_pLogoSurface, nil );


  // Updating the primary buffer with a blt
  pddsFrontBuffer := g_pDisplay.GetFrontBuffer;
  pddsBackBuffer := g_pDisplay.GetBackBuffer;
  hr := pddsFrontBuffer.Blt( nil, pddsBackBuffer, nil, DDBLT_WAIT, nil );
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  pddsFrontBuffer := nil;
  pddsBackBuffer := nil;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: RestoreSurfaces()
   Desc: Restore all the surfaces, and redraw the sprite surfaces.
-----------------------------------------------------------------------------}

function RestoreSurfaces : HRESULT;
var
  hr : HRESULT;
  //pDDPal  : IDIRECTDRAWPALETTE;
begin
  hr := g_pDisplay.GetDirectDraw.RestoreAllSurfaces;
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( TEXT("RestoreAllSurfaces"), hr );
    result := hr;
    exit;
  end;

  hr := g_pLogoSurface.DrawBitmap( PChar( '..\media\directx.bmp' ), SPRITE_DIAMETER, SPRITE_DIAMETER );
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( TEXT("DrawBitmap"), hr );
    result := hr;
    exit;
  end;

  // No need to re-create the surface, just re-draw it.
  hr := g_pTextSurface.DrawText( 0, HELPTEXT, 0, 0, RGB( 0, 0, 0 ), RGB( 255, 255, 0 ) );
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( TEXT("DrawText"), hr );
    result := hr;
    exit;
  end;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: ProcessNextFrame()
   Desc: Move the sprites, blt them to the back buffer, then
         flip or blt the back buffer to the primary buffer
-----------------------------------------------------------------------------}

function ProcessNextFrame( h_Wnd : HWND ) : HRESULT;
var
  hr : HRESULT;
  dwCurrTick : DWord;
  dwTickDiff : DWord;
  iSprite : integer;
begin
  // Figure how much time has passed since the last time
  dwCurrTick := GetTickCount;
  dwTickDiff := dwCurrTick - g_dwLastTick;

  // Don't update if no time has passed
  if ( dwTickDiff = 0 ) then
  begin
    result := DD_OK;
    exit
  end;

  g_dwLastTick := dwCurrTick;

  // Move the sprites according to how much time has passed
  for iSprite := 0 to NUM_SPRITES - 1 do
    UpdateSprite( g_Sprite[ iSprite ], dwTickDiff / 1000.0 );

  // Display the sprites on the screen
  hr := DisplayFrame;
  if ( hr <> DD_OK ) then
  begin
    if ( hr <> DDERR_SURFACELOST ) then
    begin
      FreeDirectDraw;
      //DXTRACE_ERR( 'DisplayFrame', hr );
      result := hr;
      exit;
    end;

    // The surfaces were lost so restore them
    RestoreSurfaces;
  end;

  result := DD_OK;
end;

//-----------------------------------------------------------------------------
// Name: SampleDlgProc()
// Desc: A simple dialog that has some standard controls, so you can see that
//       they update properly.
//-----------------------------------------------------------------------------

function SampleDlgProc( hDlg : HWND; AMsg : Cardinal; w_Param : Cardinal; l_Param : Integer ) : Integer; stdcall;
var
  pszCombo : array[ 0..5 ] of PChar;
  i : integer;
begin
  pszCombo[ 0 ] := 'One';
  pszCombo[ 1 ] := 'Two';
  pszCombo[ 2 ] := 'Three';
  pszCombo[ 3 ] := 'Four';
  pszCombo[ 4 ] := 'Five';
  pszCombo[ 5 ] := 'Six';

  case AMsg of
    WM_INITDIALOG :
      begin
        for i := 0 to 5 do
        begin
          SendDlgItemMessage( hDlg, IDC_COMBO1, CB_ADDSTRING, 0, Integer( pszCombo[ i ] ) );

          SendDlgItemMessage( hDlg, IDC_COMBO1, CB_SETCURSEL, 0, 0 );
          CheckDlgButton( hDlg, IDC_RADIO1, BST_CHECKED );
        end;
        result := 1;
        exit;
      end;

    WM_COMMAND :
      begin
        case LOWORD( w_Param ) of
          IDCANCEL,
            IDOK :
            begin
              EndDialog( hDlg, 1 );
              result := 1;
              exit;
            end;
        end;
      end;

    WM_MOVE :
        // The window is moving around, so re-draw the backbuffer
      DisplayFrame;

    WM_DESTROY :
      g_hWndDlg := 0;

  end;

  result := 0;
end;

{-----------------------------------------------------------------------------
   Name: WindowProc
   Desc: The Main Window Procedure
{-----------------------------------------------------------------------------}

function WindowProc( h_Wnd : THandle; aMSG : Cardinal; wParam : Cardinal; lParam : Integer ) : Integer; stdcall;
var
  h_Inst : HINST;
begin
  case aMSG of
    // Pause if minimized
    WM_COMMAND :
      begin
        case LOWORD( wParam ) of
          IDM_EXIT :
            begin
          // Received key/menu command to exit app
              PostMessage( h_Wnd, WM_CLOSE, 0, 0 );
              result := 0;
              exit;
            end;

          IDM_SHOW_DIALOG :
            begin
{$IFDEF WIN64}
              h_Inst := GetWindowLongPtr( h_Wnd, GWLP_HINSTANCE );
{$ELSE}
              h_Inst := GetWindowLong( h_Wnd, GWL_HINSTANCE );
{$ENDIF}
              g_hWndDlg := CreateDialog( h_Inst, MAKEINTRESOURCE( IDD_DIALOG_SAMPLE ),
                h_Wnd, @SampleDlgProc );
              ShowWindow( g_hWndDlg, SW_SHOWNORMAL );
            end;
        end;
      end;

    WM_SIZE :
      begin
      // Check to see if we are losing our window...
        if ( wParam = SIZE_MAXHIDE ) or ( wParam = SIZE_MINIMIZED ) then
          g_bActive := FALSE
        else
          g_bActive := TRUE;
      end;

    WM_EXITMENULOOP :
      begin
      // Ignore time spent in menu
        g_dwLastTick := GetTickCount;
      end;

    WM_EXITSIZEMOVE :
      begin
      // Ignore time spent resizing
        g_dwLastTick := GetTickCount;
      end;

    WM_SYSCOMMAND :
      begin
      // Prevent moving/sizing and power loss in fullscreen mode
        case wParam of
          SC_MOVE, SC_SIZE, SC_MAXIMIZE, SC_MONITORPOWER :
            begin
              result := 1;
              exit;
            end;
        end;
      end;

    WM_DESTROY :
      begin
      // Cleanup and close the app
        FreeDirectDraw;
        PostQuitMessage( 0 );
        result := 0;
        exit;
      end;
  end;
  Result := DefWindowProc( h_Wnd, aMSG, wParam, lParam );
end;

{-----------------------------------------------------------------------------
   Name: WinInit
   Desc: Do work required for every instance of the application:
         Create the window, initialize data
-----------------------------------------------------------------------------}

function WinInit( h_Inst : THANDLE; nCmdShow : Integer; var phWnd : THandle; var phAccel : HACCEL ) : HRESULT;
var
  h_Wnd : THandle;
  wc : TWNDCLASS;
  h_Accel : HACCEL;
begin
  // Register the window class
  //wc.cbSize        := sizeof(wc);
  wc.lpszClassName := NAME;
  wc.lpfnWndProc := @WindowProc;
  wc.style := CS_VREDRAW or CS_HREDRAW;
  wc.hInstance := h_Inst;
  wc.hIcon := LoadIcon( h_Inst, MakeIntResource( IDI_MAIN_ICON ) );
  wc.hCursor := LoadCursor( 0, IDC_ARROW );
  wc.hbrBackground := ( COLOR_WINDOW + 1 );
  wc.lpszMenuName := nil;
  wc.cbClsExtra := 0;
  wc.cbWndExtra := 0;

  if ( RegisterClass( wc ) = 0 ) then
  begin
    result := E_FAIL;
    exit;
  end;

  // Load keyboard accelerators
  h_Accel := LoadAccelerators( h_Inst, MakeIntResource( IDR_MAIN_ACCEL ) );

  // Create and show the main window
  h_Wnd := CreateWindowEx( 0,
    NAME,
    TITLE,
    WS_POPUP,
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    0,
    0,
    h_Inst,
    nil );
  if ( h_Wnd = 0 ) then
  begin
    result := E_FAIL;
    exit;
  end;

  ShowWindow( h_Wnd, nCmdShow );
  UpdateWindow( h_Wnd );

  phWnd := h_Wnd;
  phAccel := h_Accel;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: InitDirectDraw()
   Desc: Create the DirectDraw object, and init the surfaces
-----------------------------------------------------------------------------}

function InitDirectDraw( h_Wnd : HWND ) : HRESULT;
var
  hr : HRESULT;
  iSprite : integer;
  ddcaps : TDDCAPS;
begin
  // Initialize all the surfaces we need
  g_pDisplay := TDisplay.Create;

  hr := g_pDisplay.CreateFullScreenDisplay( h_Wnd, SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP );
  if ( hr <> DD_OK ) then
  begin
    MessageBox( h_Wnd, 'This display card does not support 640x480x8.', TITLE, MB_ICONERROR or MB_OK );
    result := hr;
    exit;
  end;


  // Check if the device supports DDCAPS2_CANRENDERWINDOWED.
  // If it does, then it supports GDI writing directly to the primary surface
  // Otherwise, to do GDI displaying on these cards, you you must create a
  // bitmap of the GDI window and BitBlt the bitmap to the backbuffer
  // then flip as normal. However, this sample does not show this.
  FillChar( ddcaps, SizeOf( ddcaps ), 0 );
  ddcaps.dwSize := SizeOf( ddcaps );
  g_pDisplay.GetDirectDraw.GetCaps( @ddcaps, nil );
  if ( ( ddcaps.dwCaps2 and DDCAPS2_CANRENDERWINDOWED ) = 0 ) then
  begin
    MessageBox( h_Wnd, 'This display card can not render GDI.', TITLE, MB_ICONERROR or MB_OK );
    result := E_FAIL;
    exit;
  end;

  // Create a clipper so DirectDraw will not blt over the GDI dialog
  hr := g_pDisplay.InitClipper;
  if ( hr <> DD_OK ) then
  begin
    MessageBox( h_Wnd, 'Failed on call to g_pDisplay.InitClipper.', TITLE, MB_ICONERROR or MB_OK );
    result := hr;
    exit;
  end;

  // Create a surface, and draw a bitmap resource on it.
  hr := g_pDisplay.CreateSurfaceFromBitmap( g_pLogoSurface, PAnsiChar( '..\media\directx.bmp' ) {MAKEINTRESOURCE( IDB_DIRECTX )}, SPRITE_DIAMETER, SPRITE_DIAMETER );
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( 'CreateSurfaceFromBitmap', hr )
    result := hr;
    exit
  end;

  // Create a surface, and draw text to it.
  hr := g_pDisplay.CreateSurfaceFromText( g_pTextSurface, 0, HELPTEXT, RGB( 0, 0, 0 ), RGB( 255, 255, 0 ) );
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( 'CreateSurfaceFromText', hr )
    result := hr;
    exit
  end;

  // Set the color key for the logo sprite to black
  hr := g_pLogoSurface.SetColorKey( 0 );
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( 'SetColorKey', hr )
    result := hr;
    exit
  end;

  // Init all the sprites.  All of these sprites look the same,
  // using the g_pDDSLogo surface.
  for iSprite := 0 to NUM_SPRITES - 1 do
  begin
    // Set the sprite's position and velocity
    g_Sprite[ iSprite ].fPosX := random( SCREEN_WIDTH );
    g_Sprite[ iSprite ].fPosY := random( SCREEN_HEIGHT );

    g_Sprite[ iSprite ].fVelX := 500.0 * random / RAND_MAX - 250.0;
    g_Sprite[ iSprite ].fVelY := 500.0 * random / RAND_MAX - 250.0;
  end;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: WinMain
   Desc: Entry point to the program. Initializes everything and calls
         UpdateFrame() when idle from the message pump.
-----------------------------------------------------------------------------}
var
  aMSG : MSG;
  h_Wnd : THandle;
  h_Accel : HACCEL;
  hr : HRESULT;
begin
  ZeroMemory( @g_Sprite, SizeOf( TSpriteRecord ) * NUM_SPRITES );

  random( GetTickCount );
  hr := WinInit( hInstance, SW_SHOW, h_Wnd, h_Accel );
  if ( hr <> DD_OK ) then
    exit;

  hr := InitDirectDraw( h_Wnd );
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( 'InitDirectDraw', hr );
    MessageBox( h_Wnd, 'DirectDraw init failed. ' + #13#10 + 'The sample will now exit. ', 'DirectDraw Sample', MB_ICONERROR or MB_OK );
    exit;
  end;

  // Show the dialog immediately
  g_hWndDlg := CreateDialog( hInstance, MAKEINTRESOURCE( IDD_DIALOG_SAMPLE ), h_Wnd, @SampleDlgProc );
  ShowWindow( g_hWndDlg, SW_SHOWNORMAL );
  g_dwLastTick := GetTickCount;

  while ( true ) do
  begin
    // Look for messages, if none are found then
    // update the state and display it
    if ( PeekMessage( aMSG, 0, 0, 0, PM_NOREMOVE ) ) then
    begin
      if not GetMessage( aMSG, 0, 0, 0 ) then
        // WM_QUIT was posted, so exit
        exit;

      // Translate and dispatch the message
      // Translate and dispatch the message. If the dialog is showing,
            // translate messages for it since it's a modeless dialog.
      if ( g_hWndDlg = 0 ) or ( not IsDialogMessage( g_hWndDlg, AMsg ) ) then
      begin
        if ( TranslateAccelerator( h_Wnd, h_Accel, aMsg ) = 0 ) then
        begin
          TranslateMessage( aMsg );
          DispatchMessage( aMsg );
        end;
      end;
    end
    else
    begin
      if ( g_bActive ) then
      begin
        // Move the sprites, blt them to the back buffer, then
        // flip or blt the back buffer to the primary buffer
        hr := ProcessNextFrame( h_Wnd );
        if ( hr <> DD_OK ) then
        begin
          g_pDisplay := nil;
          if ( hr = E_NOTIMPL ) then
          begin
            MessageBox( h_Wnd, 'The driver does not support waiting for vertical blank.' + #13#10 +
              'This is expected if the HW acceleration is disabled.' + #13#10 +
              'The sample will now exit.', 'DirectDraw Sample', MB_ICONERROR or MB_OK );
          end
          else
          begin
              //DXTRACE_ERR( 'ProcessNextFrame', hr );
            MessageBox( h_Wnd, 'Displaying the next frame failed.' + #13#10 + 'The sample will now exit.', 'DirectDraw Sample', MB_ICONERROR or MB_OK );
          end;
          exit;
        end;
      end
      else
      begin
        // Go to sleep if we have nothing else to do
        WaitMessage;

        // Ignore time spent inactive
        g_dwLastTick := GetTickCount;
      end;
    end;
  end;
end.
