program GammaControl_api;
{******************************************************************}
{                                                                  }
{       Borland Delphi DirectX 8 SDK Examples                      }
{       Conversion of the Gamma Control Example                    }
{                                                                  }
{ Portions created by Microsoft are                                }
{ Copyright (C) 1995-2000 Microsoft Corporation.                   }
{ All Rights Reserved.                                             }
{                                                                  }
{ The original files are : GammaControl.h,                         }
{                          GammaControl.cpp                        }
{ The original Pascal code is : GammaControl_api.pas               }
{ The initial developer of the Pascal code is : Dominique Louis    }
{ ( Dominique@SavageSoftware.com.au )                              }
{                                                                  }
{ Portions created by Dominique Louis are                          }
{ Copyright (C) 2000 - 2001 Dominique Louis.                       }
{                                                                  }
{ Contributor(s)                                                   }
{ --------------                                                   }
{ <Contributer Name> ( contributer@sp.sw )                         }
{                                                                  }
{ Obtained through:                                                }
{ Joint Endeavour of Delphi Innovators ( Project JEDI )            }
{                                                                  }
{ You may retrieve the latest version of this file at the Project  }
{ JEDI home page, located at http://delphi-jedi.org                }
{                                                                  }
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/NPL/NPL-1_1Final.html                     }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{ Description                                                      }
{ -----------                                                      }
{   GammaControl demonstrates how to use IDirectDrawGammaControl to}
{   adjust how bright the system displays the DirectDraw surface.  }
{                                                                  }
{ Requires                                                         }
{ --------                                                         }
{   Eric Unger and Tim Baumgarten's conversions of the DirectX     }
{   headers. They are available from...                            }
{   http://delphi-jedi.org/DelphiGraphics/ .                       }
{                                                                  }
{                                                                  }
{ Programming Notes                                                }
{ -----------------                                                }
{   For details on how to setup a full-screen DirectDraw app, see  }
{   the FullScreenMode sample.                                     }
{                                                                  }
{   To adjust the gamma, first check to see if the device supports }
{   it.  Call IDirectDraw::GetCaps to check if DDCAPS2_PRIMARYGAMMA}
{   is set.  If it is, then you can set the gamma ramp by calling  }
{   IDirectDrawGammaControl::SetGammaRamp.                         }
{                                                                  }
{   For simplicity this sample creates a gamma ramp which is linear}
{   for all color components.  The ramp runs from 0 at index 0     }
{   linearly up to a user defined number at index 255 for each of  }
{   the color components.  However, the ramp does not need to be   }
{   linear.                                                        }
{                                                                  }
{ Revision History                                                 }
{ ----------------                                                 }
{   2000-12-01 - DL  Initial translation.                          }
{                                                                  }
{                                                                  }
{******************************************************************}

uses
  Windows,
  SysUtils,
  Messages,
  DirectDraw,
  DDUtil in '..\..\Common\DDUtil.pas';

{$R *.RES}
const
  NAME : PChar = 'GammaControl';
  TITLE : PChar = 'DirectDraw GammaControl Example';
  HELPTEXT : PChar = 'Adjust the gamma level by clicking either mouse button. Press Escape to quit.';
  GAMMATEXT : PChar = 'Displaying with linear gamma ramp between 0 and %0.5d';

  IDI_MAIN_ICON = 101;
  IDR_MAIN_ACCEL = 103;
  IDB_DIRECTX = 107;
  IDM_EXIT = 1001;

  SCREEN_WIDTH = 640;
  SCREEN_HEIGHT = 480;
  SCREEN_BPP = 32;

  NUM_COLOR_BARS = 63;

var
  g_pDisplay : TDisplay = nil;
  g_pTextSurface : TSurface = nil;
  g_pGammaTextSurface : TSurface = nil;
  g_pGammaControl : IDIRECTDRAWGAMMACONTROL = nil;
  g_bActive : Boolean = False; // Is application active?
  g_lGammaRamp : LongInt = 256;


{-----------------------------------------------------------------------------
  Name: FreeDirectDraw
  Desc: Release all the DirectDraw objects
-----------------------------------------------------------------------------}

procedure FreeDirectDraw;
begin
  g_pTextSurface.Free;
  g_pTextSurface := nil;
  g_pGammaTextSurface.Free;
  g_pGammaTextSurface := nil;
  g_pDisplay.Free;
  g_pDisplay := nil;
end;

{-----------------------------------------------------------------------------
   Name: DisplayFrame()
   Desc: Blts a the color bars to the back buffer, then flips the
         back buffer onto the primary buffer.
-----------------------------------------------------------------------------}

function DisplayFrame : HRESULT;
var
  hr : HRESULT;
  ddsd : TDDSURFACEDESC2;
  ddbltfx : TDDBLTFX;
  rctDest : TRECT;
  dwRShift, dwGShift, dwBShift : DWORD;
  dwRColorLevels, dwGColorLevels, dwBColorLevels : DWORD;
  dwBits : DWORD;
  dwColor : DWORD;
  pFrontBuffer : IDIRECTDRAWSURFACE7;
  pBackBuffer : IDIRECTDRAWSURFACE7;
  i : integer;
begin
  if g_pDisplay = nil then
  begin
    result := DD_OK;
    exit;
  end;

  pFrontBuffer := g_pDisplay.GetFrontBuffer;
  pBackBuffer := g_pDisplay.GetBackBuffer;

  FillChar( ddsd, SizeOf( ddsd ), 0 );
  ddsd.dwSize := SizeOf( ddsd );
  hr := pFrontBuffer.GetSurfaceDesc( ddsd );
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  // Fill the back buffer with black, ignoring errors until the flip
  g_pDisplay.Clear( 0 );

  g_pTextSurface.GetBitMaskInfo( ddsd.ddpfPixelFormat.dwRBitMask, dwRShift, dwBits );
  g_pTextSurface.GetBitMaskInfo( ddsd.ddpfPixelFormat.dwGBitMask, dwGShift, dwBits );
  g_pTextSurface.GetBitMaskInfo( ddsd.ddpfPixelFormat.dwBBitMask, dwBShift, dwBits );

  dwRColorLevels := ddsd.ddpfPixelFormat.dwRBitMask shr dwRShift;
  dwGColorLevels := ddsd.ddpfPixelFormat.dwGBitMask shr dwGShift;
  dwBColorLevels := ddsd.ddpfPixelFormat.dwBBitMask shr dwBShift;

  FillChar( ddbltfx, SizeOf( ddbltfx ), 0 );
  ddbltfx.dwSize := sizeof( ddbltfx );

  for i := 0 to NUM_COLOR_BARS do
  begin
    rctDest.left := Trunc( SCREEN_WIDTH / ( NUM_COLOR_BARS + 1 ) * ( i + 0 ) );
    rctDest.right := Trunc( SCREEN_WIDTH / ( NUM_COLOR_BARS + 1 ) * ( i + 1 ) );

    // Figure out the color for the red color bar
    dwColor := Round( dwRColorLevels * i / NUM_COLOR_BARS );
    ddbltfx.dwFillColor := dwColor shl dwRShift;

    rctDest.top := Trunc( SCREEN_HEIGHT / 3 * 0 );
    rctDest.bottom := Trunc( SCREEN_HEIGHT / 3 * 1 );
    pBackBuffer.Blt( @rctDest, nil, nil, DDBLT_COLORFILL or DDBLT_WAIT, @ddbltfx );

    // Figure out the color for the green color bar
    dwColor := Round( dwGColorLevels * i / NUM_COLOR_BARS );
    ddbltfx.dwFillColor := dwColor shl dwGShift;

    rctDest.top := Trunc( SCREEN_HEIGHT / 3 * 1 );
    rctDest.bottom := Trunc( SCREEN_HEIGHT / 3 * 2 );
    pBackBuffer.Blt( @rctDest, nil, nil, DDBLT_COLORFILL or DDBLT_WAIT, @ddbltfx );

    // Figure out the color for the blue color bar
    dwColor := Trunc( dwBColorLevels * i / NUM_COLOR_BARS );
    ddbltfx.dwFillColor := dwColor shl dwBShift;

    rctDest.top := Trunc( SCREEN_HEIGHT / 3 * 2 );
    rctDest.bottom := Trunc( SCREEN_HEIGHT / 3 * 3 );
    pBackBuffer.Blt( @rctDest, nil, nil, DDBLT_COLORFILL or DDBLT_WAIT, @ddbltfx );
  end;

  // Blt the help text on the backbuffer, ignoring errors until the flip
  g_pDisplay.Blt( 10, 10, g_pTextSurface, nil );

  // Blt the Gamma text on the backbuffer, ignoring errors until the flip
  g_pDisplay.Blt( 10, 30, g_pGammaTextSurface, nil );

  // Flip or blt the back buffer onto the primary buffer
  hr := g_pDisplay.Flip;
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  result := DD_OK;
end;

//-----------------------------------------------------------------------------
// Name: DrawGammaText()
// Desc:
//-----------------------------------------------------------------------------

function DrawGammaText : HResult;
var
  hr : HRESULT;
  strGammaText : string;
begin
  strGammaText := Format( GAMMATEXT, [ g_lGammaRamp * 256 ] );
  hr := g_pGammaTextSurface.DrawText( 0, strGammaText, 0, 0, RGB( 0, 0, 0 ), RGB( 255, 255, 0 ) );
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: RestoreSurfaces()
   Desc: Restore all the surfaces, and redraw the sprite surfaces.
-----------------------------------------------------------------------------}

function RestoreSurfaces : HRESULT;
var
  hr : HRESULT;
begin
  hr := g_pDisplay.GetDirectDraw.RestoreAllSurfaces;
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( TEXT("RestoreAllSurfaces"), hr );
    result := hr;
    exit;
  end;

  // No need to re-create the surface, just re-draw it.
  hr := g_pTextSurface.DrawText( 0, HELPTEXT, 0, 0, RGB( 0, 0, 0 ), RGB( 255, 255, 0 ) );
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( TEXT("DrawText"), hr );
    result := hr;
    exit;
  end;

  DrawGammaText;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: ProcessNextFrame()
   Desc: Move the sprites, blt them to the back buffer, then
         flip or blt the back buffer to the primary buffer
-----------------------------------------------------------------------------}

function ProcessNextFrame( h_Wnd : HWND ) : HRESULT;
var
  hr : HRESULT;
begin

  // Display the sprites on the screen
  hr := DisplayFrame;
  if ( hr <> DD_OK ) then
  begin
    if ( hr <> DDERR_SURFACELOST ) then
    begin
      FreeDirectDraw;
      //DXTRACE_ERR( 'DisplayFrame', hr );
      result := hr;
      exit;
    end;

    // The surfaces were lost so restore them
    RestoreSurfaces;
  end;

  result := DD_OK;
end;

//-----------------------------------------------------------------------------
// Name: UpdateGammaRamp()
// Desc:
//-----------------------------------------------------------------------------

function UpdateGammaRamp : HRESULT;
var
  hr : HRESULT;
  ddgr : TDDGammaRamp;
  dwGamma : Word;
  iColor : Integer;
begin
  FillChar( ddgr, SizeOf( ddgr ), 0 );
  hr := g_pGammaControl.GetGammaRamp( 0, ddgr );
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  dwGamma := 0;

  for iColor := 0 to 256 - 1 do
  begin
    ddgr.red[ iColor ] := dwGamma;
    ddgr.green[ iColor ] := dwGamma;
    ddgr.blue[ iColor ] := dwGamma;

    dwGamma := dwGamma + g_lGammaRamp;
  end;

  hr := g_pGammaControl.SetGammaRamp( 0, ddgr );
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  hr := DrawGammaText;
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: WindowProc
   Desc: The Main Window Procedure
{-----------------------------------------------------------------------------}

function WindowProc( h_Wnd : THandle; aMSG : Cardinal; wParam : Cardinal; lParam : Integer ) : Integer; stdcall;
begin
  case aMSG of

    WM_LBUTTONDOWN :
      begin
        g_lGammaRamp := g_lGammaRamp - 8;
        if ( g_lGammaRamp < 0 ) then
          g_lGammaRamp := 0;
        if ( UpdateGammaRamp <> DD_OK ) then
        begin
          g_pDisplay.GetDirectDraw.FlipToGDISurface;

          g_pDisplay := nil;

          MessageBox( h_Wnd, 'Failed setting the new gamma level. ' + #13#10 +
            'The sample will now exit.', TITLE, MB_ICONERROR or MB_OK );

          PostQuitMessage( 0 );
        end;
      end;

    WM_RBUTTONDOWN :
      begin
        g_lGammaRamp := g_lGammaRamp + 8;
        if ( g_lGammaRamp > 256 ) then
          g_lGammaRamp := 256;
        if ( UpdateGammaRamp <> DD_OK ) then
        begin
          g_pDisplay.GetDirectDraw.FlipToGDISurface;

          g_pDisplay := nil;

          MessageBox( h_Wnd, 'Failed setting the new gamma level. ' + #13#10 +
            'The sample will now exit.', TITLE, MB_ICONERROR or MB_OK );

          PostQuitMessage( 0 );
        end;
      end;

    // Pause if minimized
    WM_COMMAND :
      begin
        case LOWORD( wParam ) of
          IDM_EXIT :
            begin
          // Received key/menu command to exit app
              PostMessage( h_Wnd, WM_CLOSE, 0, 0 );
              result := 0;
              exit;
            end;
        end;
      end;

    WM_SETCURSOR :
      begin
      // Hide the cursor if in fullscreen
        SetCursor( 0 );
        result := 1;
        exit;
      end;

    WM_SIZE :
      begin
      // Check to see if we are losing our window...
        if ( wParam = SIZE_MAXHIDE ) or ( wParam = SIZE_MINIMIZED ) then
          g_bActive := FALSE
        else
          g_bActive := TRUE;
      end;

    WM_SYSCOMMAND :
      begin
      // Prevent moving/sizing and power loss in fullscreen mode
        case wParam of
          SC_MOVE, SC_SIZE, SC_MAXIMIZE, SC_MONITORPOWER :
            begin
              result := 1;
              exit;
            end;
        end;
      end;

    WM_DESTROY :
      begin
      // Cleanup and close the app
        FreeDirectDraw;
        PostQuitMessage( 0 );
        result := 0;
        exit;
      end;
  end;
  Result := DefWindowProc( h_Wnd, aMSG, wParam, lParam );
end;

{-----------------------------------------------------------------------------
   Name: WinInit
   Desc: Do work required for every instance of the application:
         Create the window, initialize data
-----------------------------------------------------------------------------}

function WinInit( h_Inst : THANDLE; nCmdShow : Integer; var phWnd : THandle; var phAccel : HACCEL ) : HRESULT;
var
  h_Wnd : THandle;
  wc : TWNDCLASS;
  h_Accel : HACCEL;
begin
  // Register the window class
  //wc.cbSize        := sizeof(wc);
  wc.lpszClassName := NAME;
  wc.lpfnWndProc := @WindowProc;
  wc.style := CS_VREDRAW or CS_HREDRAW;
  wc.hInstance := h_Inst;
  wc.hIcon := LoadIcon( h_Inst, MakeIntResource( IDI_MAIN_ICON ) );
  wc.hCursor := LoadCursor( 0, IDC_ARROW );
  wc.hbrBackground := ( COLOR_WINDOW + 1 );
  wc.lpszMenuName := nil;
  wc.cbClsExtra := 0;
  wc.cbWndExtra := 0;

  if ( RegisterClass( wc ) = 0 ) then
  begin
    result := E_FAIL;
    exit;
  end;

  // Load keyboard accelerators
  h_Accel := LoadAccelerators( h_Inst, MakeIntResource( IDR_MAIN_ACCEL ) );

  // Create and show the main window
  h_Wnd := CreateWindowEx( 0,
    NAME,
    TITLE,
    WS_POPUP,
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    0,
    0,
    h_Inst,
    nil );
  if ( h_Wnd = 0 ) then
  begin
    result := E_FAIL;
    exit;
  end;

  ShowWindow( h_Wnd, nCmdShow );
  UpdateWindow( h_Wnd );

  phWnd := h_Wnd;
  phAccel := h_Accel;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: InitDirectDraw()
   Desc: Create the DirectDraw object, and init the surfaces
-----------------------------------------------------------------------------}

function InitDirectDraw( h_Wnd : HWND ) : HRESULT;
var
  hr : HRESULT;
  strGammaText : string;
begin
  // Initialize all the surfaces we need
  g_pDisplay := TDisplay.Create;

  hr := g_pDisplay.CreateFullScreenDisplay( h_Wnd, SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP );
  if ( hr <> DD_OK ) then
  begin
    MessageBox( h_Wnd, 'This display card does not support 640x480x32.', TITLE, MB_ICONERROR or MB_OK );
    result := hr;
    exit
  end;

  // Create a surface, and draw text to it.
  hr := g_pDisplay.CreateSurfaceFromText( g_pTextSurface, 0, HELPTEXT, RGB( 0, 0, 0 ), RGB( 255, 255, 0 ) );
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( 'CreateSurfaceFromText', hr )
    result := hr;
    exit
  end;

  strGammaText := Format( GAMMATEXT, [ g_lGammaRamp * 256 ] );
  // Create a surface, and draw text to it.
  hr := g_pDisplay.CreateSurfaceFromText( g_pGammaTextSurface, 0, strGammaText, RGB( 0, 0, 0 ), RGB( 255, 255, 0 ) );
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( 'CreateSurfaceFromText', hr )
    result := hr;
    exit
  end;

  result := DD_OK;
end;

//-----------------------------------------------------------------------------
// Name: HasGammaSupport()
// Desc: Returns TRUE if the device supports gamma control, FALSE otherwise
//-----------------------------------------------------------------------------

function HasGammaSupport : Boolean;
var
  ddcaps : TDDCAPS;
begin
  // Get driver capabilities to determine gamma support.
  FillChar( ddcaps, SizeOf( ddcaps ), 0 );
  ddcaps.dwSize := SizeOf( ddcaps );

  g_pDisplay.GetDirectDraw.GetCaps( @ddcaps, nil );

  // Does the driver support gamma?
  // The DirectDraw emulation layer does not support overlays
  // so gamma related APIs will fail without hardware support.
  if ( ( ddcaps.dwCaps2 and DDCAPS2_PRIMARYGAMMA ) <> 0 ) then
    result := True
  else
    result := False;
end;

{-----------------------------------------------------------------------------
   Name: WinMain
   Desc: Entry point to the program. Initializes everything and calls
         UpdateFrame() when idle from the message pump.
-----------------------------------------------------------------------------}
var
  aMSG : MSG;
  h_Wnd : THandle;
  h_Accel : HACCEL;
  hr : HRESULT;
begin
  hr := WinInit( hInstance, SW_SHOW, h_Wnd, h_Accel );
  if ( hr <> DD_OK ) then
    exit;

  hr := InitDirectDraw( h_Wnd );
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( 'InitDirectDraw', hr );
    MessageBox( h_Wnd, 'DirectDraw init failed. ' +
      #13#10 + 'The sample will now exit. ', TITLE,
      MB_ICONERROR or MB_OK );
    exit;
  end;

  if not HasGammaSupport then
  begin
    g_pDisplay.GetDirectDraw.FlipToGDISurface;

    g_pDisplay := nil;

    MessageBox( h_Wnd, 'The primary surface does not support gamma control. ' +
      #13#10 + 'The sample will now exit. ', TITLE,
      MB_ICONERROR or MB_OK );
    exit;
  end;

  hr := g_pDisplay.GetFrontBuffer.QueryInterface( IID_IDirectDrawGammaControl, g_pGammaControl );
  if ( hr <> DD_OK ) then
  begin
    g_pDisplay.GetDirectDraw.FlipToGDISurface;

    g_pDisplay := nil;

    MessageBox( h_Wnd, 'The primary surface does not support IDirectDrawGammaControl. ' +
      #13#10 + 'The sample will now exit. ', TITLE,
      MB_ICONERROR or MB_OK );
    exit;
  end;

  while ( true ) do
  begin
    // Look for messages, if none are found then
    // update the state and display it
    if ( PeekMessage( aMSG, 0, 0, 0, PM_NOREMOVE ) ) then
    begin
      if not GetMessage( aMSG, 0, 0, 0 ) then
        // WM_QUIT was posted, so exit
        exit;

      // Translate and dispatch the message
      if ( TranslateAccelerator( h_Wnd, h_Accel, aMsg ) = 0 ) then
      begin
        TranslateMessage( aMsg );
        DispatchMessage( aMsg );
      end;
    end
    else
    begin
      if ( g_bActive ) then
      begin
        // Move the sprites, blt them to the back buffer, then
        // flip or blt the back buffer to the primary buffer
        hr := ProcessNextFrame( h_Wnd );
        if ( hr <> DD_OK ) then
        begin
          g_pDisplay := nil;
          if ( hr = E_NOTIMPL ) then
          begin
            MessageBox( h_Wnd, 'The driver does not support waiting for vertical blank.' + #13#10 +
              'This is expected if the HW acceleration is disabled.' + #13#10 +
              'The sample will now exit.', TITLE, MB_ICONERROR or MB_OK );
          end
          else
          begin
              //DXTRACE_ERR( 'ProcessNextFrame', hr );
            MessageBox( h_Wnd, 'Displaying the next frame failed.' + #13#10 + 'The sample will now exit.', TITLE, MB_ICONERROR or MB_OK );
          end;
          exit;
        end;
      end
      else
      begin
        // Go to sleep if we have nothing else to do
        WaitMessage;
      end;
    end;
  end;
end.

