program OverlayAnimate_api;
{******************************************************************}
{                                                                  }
{       Borland Delphi DirectX 8 SDK Examples                      }
{       Conversion of the Overlay Animate Example                  }
{                                                                  }
{ Portions created by Microsoft are                                }
{ Copyright (C) 1995-2000 Microsoft Corporation.                   }
{ All Rights Reserved.                                             }
{                                                                  }
{ The original files are : OverlayAnimate.h,                       }
{                          OverlayAnimate.cpp                      }
{ The original Pascal code is : OverlayAnimate_api.pas             }
{ The initial developer of the Pascal code is : Dominique Louis    }
{ ( Dominique@SavageSoftware.com.au )                              }
{                                                                  }
{ Portions created by Dominique Louis are                          }
{ Copyright (C) 2000 - 2001 Dominique Louis.                       }
{                                                                  }
{ Contributor(s)                                                   }
{ --------------                                                   }
{ <Contributer Name> ( contributer@sp.sw )                         }
{                                                                  }
{ Obtained through:                                                }
{ Joint Endeavour of Delphi Innovators ( Project JEDI )            }
{                                                                  }
{ You may retrieve the latest version of this file at the Project  }
{ JEDI home page, located at http://delphi-jedi.org                }
{                                                                  }
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/NPL/NPL-1_1Final.html                     }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{ Description                                                      }
{ -----------                                                      }
{   OverlayAnimate demonstrates how to use DirectDraw overlays.    }
{                                                                  }
{ Requires                                                         }
{ --------                                                         }
{   Eric Unger and Tim Baumgarten's conversions of the DirectX     }
{   headers. They are available from...                            }
{   http://delphi-jedi.org/DelphiGraphics/ .                       }
{                                                                  }
{                                                                  }
{ Programming Notes                                                }
{ -----------------                                                }
{   For details on how to setup a windowed mode DirectDraw app, see the WindowedMode
  sample.

  To use overlays in general do the following steps in addition to those needed to
  author a windowed mode DirectDraw app:
    1. Check to see if hardware supports overlays - check IDirectDraw::GetCaps
       for DDCAPS_OVERLAY.
    2. Size the window to meet the hardware overlay size restrictions.
    3. Create an overlay surface (create it with 1 backbuffer if needed), and
       set its pixel format to a desired format that is supported by the device.
    4. Set the dest color key on the overlay to the background color of the window.
       Be sure to choose a color for the background of the window that Windows typically
       does not use otherwise, the overlay will be drawn on top of overlapping windows.
    5. Call UpdateOverlay to display or hide the overlay on the desktop.  WM_PAINT is
       a good place for this.
    6. When WM_SIZE or WM_MOVE is sent, then update the src and dest rects and
       check to make sure that they are within the hardware limits.

  To animate the overlay, instead of drawing to the off-screen plain back buffer as
  in windowed mode case, just draw to the the overlay's backbuffer then flip it.}
{                                                                  }
{ Revision History                                                 }
{ ----------------                                                 }
{   2000-12-01 - DL  Initial translation.                          }
{                                                                  }
{                                                                  }
{******************************************************************}

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  DirectDraw,
  DDUtil in '..\..\Common\DDUtil.pas';

{$R *.res}

const
  NAME: PChar = 'OverlayAnimate';
  TITLE: PChar = 'DirectDraw Overlay Animation Example';
  IDI_MAIN_ICON = 101;
  IDR_MENU = 102;
  IDR_MAIN_ACCEL = 103;
  IDB_ANIMATE_SHEET = 107;
  IDM_EXIT = 1001;

  SCREEN_WIDTH = 128;
  SCREEN_HEIGHT = 128;
  SCREEN_BPP = 16;
  SPRITE_DIAMETER = 48;
  NUM_FRAMES = 30;

  HELPTEXT = 'Press Escape to quit.';

type
  PSpriteRecord = ^TSpriteRecord;
  TSpriteRecord = packed record
    fPosX: Single;
    fPosY: Single;
    fVelX: Single;
    fVelY: Single;
  end;

var
  g_pDD: IDIRECTDRAW7 = nil;
  g_pDDSPrimary: IDIRECTDRAWSURFACE7 = nil;
  g_pDDSOverlay: IDIRECTDRAWSURFACE7 = nil;
  g_pDDSOverlayBack: IDIRECTDRAWSURFACE7 = nil;
  g_pDDSAnimationSheet: IDIRECTDRAWSURFACE7 = nil;
  g_OverlayFX: TDDOVERLAYFX;
  g_dwOverlayFlags: DWORD = 0;
  g_dwFrame: DWORD = 0;
  g_ddcaps: TDDCAPS;
  g_bActive: BOOLean = FALSE;
  g_rcSrc: TRect;
  g_rcDst: TRect;
  g_dwXRatio: DWORD;
  g_dwYRatio: DWORD;
  h_Wnd: THandle;

  // This will be used as the color key, so try to make it something
  // that doesn't appear in the source image.
  g_dwBackgroundColor: TColorRef;

  {-----------------------------------------------------------------------------
    Name: FreeDirectDraw
    Desc: Release all the DirectDraw objects
  -----------------------------------------------------------------------------}

procedure FreeDirectDraw;
begin
  g_pDDSPrimary := nil;
  g_pDDSOverlayBack := nil;
  g_pDDSOverlay := nil;
  g_pDDSAnimationSheet := nil;
  g_pDD := nil;
end;


{-----------------------------------------------------------------------------
   Name: InitDirectDraw()
   Desc: Create the DirectDraw object, and init the surfaces
-----------------------------------------------------------------------------}
function InitDirectDraw(h_Wnd: HWND): HRESULT;
var
  hr: HRESULT;
  ddsd: TDDSURFACEDESC2;
begin
  // Create the main DirectDraw object
  hr := DirectDrawCreateEx(nil, g_pDD, IID_IDirectDraw7, nil);
  if (hr <> DD_OK) then
  begin
    MessageBox(h_Wnd, 'Failed initializing DirectDraw.', TITLE, MB_ICONERROR or
      MB_OK);
    result := hr;
    exit;
  end;

  // Request normal cooperative level to put us in windowed mode
  hr := g_pDD.SetCooperativeLevel(h_Wnd, DDSCL_NORMAL);
  if (hr <> DD_OK) then
  begin
    MessageBox(h_Wnd, 'Failed to Set Cooperative Level.', TITLE, MB_ICONERROR or
      MB_OK);
    result := hr;
    exit;
  end;

  // Get driver capabilities to determine Overlay support.
  ZeroMemory(@g_ddcaps, sizeof(g_ddcaps));
  g_ddcaps.dwSize := sizeof(g_ddcaps);

  hr := g_pDD.GetCaps(@g_ddcaps, nil);
  if (hr <> DD_OK) then
  begin
    MessageBox(h_Wnd, 'Failed to GetCaps.', TITLE, MB_ICONERROR or
      MB_OK);
    result := hr;
    exit;
  end;

  // Create the primary surface, which in windowed mode is the desktop.
  ZeroMemory(@ddsd, sizeof(ddsd));
  ddsd.dwSize := sizeof(ddsd);
  ddsd.dwFlags := DDSD_CAPS;
  ddsd.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE;

  hr := g_pDD.CreateSurface(ddsd, g_pDDSPrimary, nil);
  if (hr <> DD_OK) then
  begin
    MessageBox(h_Wnd, 'Failed to GetCaps.', TITLE, MB_ICONERROR or
      MB_OK);
    result := hr;
    exit;
  end;
  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: DisplayFrame()
   Desc: Blts a the sprites to the back buffer, then it blts or flips the
         back buffer onto the primary buffer.
-----------------------------------------------------------------------------}

function DisplayFrame: HRESULT;
var
  hr: HRESULT;
  rcSrc: TRect;
begin
  rcSrc.left := (g_dwFrame mod 5) * SPRITE_DIAMETER;
  rcSrc.top := (g_dwFrame div 5) * SPRITE_DIAMETER;
  rcSrc.right := rcSrc.left + SPRITE_DIAMETER;
  rcSrc.bottom := rcSrc.top + SPRITE_DIAMETER;

  g_pDDSOverlayBack.Blt(nil, g_pDDSAnimationSheet, @rcSrc, DDBLT_WAIT, nil);

  hr := g_pDDSOverlay.Flip(nil, DDFLIP_WAIT);
  if (hr <> DD_OK) then
  begin
    result := hr;
    exit;
  end;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: RestoreSurfaces()
   Desc: Restore all the surfaces, and redraw the sprite surfaces.
-----------------------------------------------------------------------------}

function RestoreSurfaces: HRESULT;
var
  hr: HRESULT;
  animateSurface: TSurface;
begin
  hr := g_pDD.RestoreAllSurfaces;
  if (hr <> DD_OK) then
  begin
    result := hr;
    exit;
  end;

  // No need to re-create the surface, just re-draw it.
  animateSurface := TSurface.Create;
  try
    animateSurface.CreateSurface(g_pDDSAnimationSheet);
    hr := animateSurface.DrawBitmap('../Media/animate.bmp'{MAKEINTRESOURCE(IDB_ANIMATE_SHEET)},
      SPRITE_DIAMETER * 5, SPRITE_DIAMETER * 6);
    if (hr <> DD_OK) then
    begin
      result := hr;
      exit;
    end;
  finally
    animateSurface.Free;
  end;

  result := DD_OK;
end;

//-----------------------------------------------------------------------------
// Name: CreateDirectDrawSurfaces()
// Desc: Creates the DirectDraw surfaces
//-----------------------------------------------------------------------------

function CreateDirectDrawSurfaces(Window: THandle): HResult;
var
  ddsd: TDDSURFACEDESC2;
  ddpfOverlayFormat: TDDPIXELFORMAT;
  ddscaps: TDDSCAPS2;
  hr: HRESULT;
  pClipper: IDIRECTDRAWCLIPPER;
  // Using a color key will clip the overlay
  // when the mouse or other windows go on top of us.
  dwDDSColor: DWORD;

  // The color key can be any color, but a near black (not exactly) allows
  // the cursor to move around on the window without showing off the
  // color key, and also clips windows with exactly black text.
  frontSurface: TSurface;
  animateSurface: TSurface;
begin
  // Release any previous surfaces
  g_pDDSOverlay := nil;

  // Set the overlay format to 16 bit RGB 5:6:5
  ZeroMemory(@ddpfOverlayFormat, sizeof(ddpfOverlayFormat));
  ddpfOverlayFormat.dwSize := sizeof(ddpfOverlayFormat);
  ddpfOverlayFormat.dwFlags := DDPF_RGB;
  ddpfOverlayFormat.dwRGBBitCount := 16;
  ddpfOverlayFormat.dwRBitMask := $F800;
  ddpfOverlayFormat.dwGBitMask := $07E0;
  ddpfOverlayFormat.dwBBitMask := $001F;

  // Setup the overlay surface's attributes in the surface descriptor
  ZeroMemory(@ddsd, sizeof(ddsd));
  ddsd.dwSize := sizeof(ddsd);
  ddsd.dwFlags := DDSD_CAPS or DDSD_HEIGHT or DDSD_WIDTH or
    DDSD_BACKBUFFERCOUNT or DDSD_PIXELFORMAT;
  ddsd.ddsCaps.dwCaps := DDSCAPS_OVERLAY or DDSCAPS_FLIP or
    DDSCAPS_COMPLEX or DDSCAPS_VIDEOMEMORY;
  ddsd.dwBackBufferCount := 1;
  ddsd.dwWidth := SCREEN_WIDTH;
  ddsd.dwHeight := SCREEN_HEIGHT;
  ddsd.ddpfPixelFormat := ddpfOverlayFormat; // Use 16 bit RGB 5:6:5 pixel format

  // Attempt to create the surface with theses settings
  hr := g_pDD.CreateSurface(ddsd, g_pDDSOverlay, nil);
  if (hr <> DD_OK) then
  begin
    result := hr;
    exit;
  end;

  ZeroMemory(@ddscaps, sizeof(ddscaps));
  ddscaps.dwCaps := DDSCAPS_BACKBUFFER;
  hr := g_pDDSOverlay.GetAttachedSurface(ddscaps, g_pDDSOverlayBack);
  if (hr <> DD_OK) then
  begin
    result := hr;
    exit;
  end;

  // Setup effects structure
  ZeroMemory(@g_OverlayFX, sizeof(g_OverlayFX));
  g_OverlayFX.dwSize := sizeof(g_OverlayFX);

  // Setup overlay flags.
  g_dwOverlayFlags := DDOVER_SHOW;

  // Check for destination color keying capability
  if (g_ddcaps.dwCKeyCaps and DDCKEYCAPS_DESTOVERLAY) <> 0 then
  begin
    frontSurface := TSurface.Create;
    try
      frontSurface.CreateSurface(g_pDDSPrimary);
      dwDDSColor := frontSurface.ConvertGDIColor(g_dwBackgroundColor);
    finally
      frontSurface.Free;
    end;
    g_OverlayFX.dckDestColorkey.dwColorSpaceLowValue := dwDDSColor;
    g_OverlayFX.dckDestColorkey.dwColorSpaceHighValue := dwDDSColor;
    g_dwOverlayFlags := g_dwOverlayFlags or DDOVER_DDFX or
      DDOVER_KEYDESTOVERRIDE;
  end
  else
  begin
    pClipper := nil;

    // If not, we'll setup a clipper for the window.  This will fix the
    // problem on a few video cards - but the ones that don't shouldn't
    // care.
    hr := g_pDD.CreateClipper(0, pClipper, nil);
    if (hr <> DD_OK) then
    begin
      result := hr;
      exit;
    end;

    hr := pClipper.SetHWnd(0, h_Wnd);
    if (hr <> DD_OK) then
    begin
      result := hr;
      exit;
    end;

    hr := g_pDDSPrimary.SetClipper(pClipper);
    if (hr <> DD_OK) then
    begin
      result := hr;
      exit;
    end;

    pClipper := nil;
  end;

  ZeroMemory(@ddsd, sizeof(ddsd));
  ddsd.dwSize := sizeof(ddsd);
  ddsd.dwFlags := DDSD_CAPS or DDSD_HEIGHT or DDSD_WIDTH or DDSD_PIXELFORMAT;
  ddsd.ddsCaps.dwCaps := DDSCAPS_OFFSCREENPLAIN;
  ddsd.dwWidth := SPRITE_DIAMETER * 5;
  ddsd.dwHeight := SPRITE_DIAMETER * 6;
  ddsd.ddpfPixelFormat := ddpfOverlayFormat; // Use 16 bit RGB 5:6:5 pixel format

  // Attempt to create the surface with theses settings
  hr := g_pDD.CreateSurface(ddsd, g_pDDSAnimationSheet, nil);
  if (hr <> DD_OK) then
  begin
    result := hr;
    exit;
  end;

  animateSurface := TSurface.Create;
  try
    animateSurface.CreateSurface(g_pDDSAnimationSheet);
    hr := animateSurface.DrawBitmap('../Media/animate.bmp'{MAKEINTRESOURCE(IDB_ANIMATE_SHEET)},
      SPRITE_DIAMETER * 5, SPRITE_DIAMETER * 6);
    if (hr <> DD_OK) then
    begin
      result := hr;
      exit;
    end;
  finally
    animateSurface.Free;
  end;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: ProcessNextFrame()
   Desc: Move the sprites, blt them to the back buffer, then
         flip or blt the back buffer to the primary buffer
-----------------------------------------------------------------------------}

function ProcessNextFrame(h_Wnd: HWND): HRESULT;
const
  s_dwFrameSkip: integer = 0;
var
  hr: HRESULT;
begin
  //s_dwFrameSkip := 0;

  // Only advance the frame animation number every 5th frame
  inc(s_dwFrameSkip);
  s_dwFrameSkip := s_dwFrameSkip mod 5;

  if (s_dwFrameSkip = 0) then
  begin
    inc(g_dwFrame);
    g_dwFrame := g_dwFrame mod NUM_FRAMES;
  end;

  // Check the cooperative level before rendering
  hr := g_pDD.TestCooperativeLevel;
  if (hr <> DD_OK) then
  begin
    case hr of

      DDERR_EXCLUSIVEMODEALREADYSET:
        begin
          // Do nothing because some other app has exclusive mode
          Sleep(10);
          result := DD_OK;
          exit;
        end;

      DDERR_WRONGMODE:
        begin
          // The display mode changed on us. Update the
          // DirectDraw surfaces accordingly
          result := CreateDirectDrawSurfaces(h_Wnd);
          exit;
        end;
    end;
    result := hr;
    exit;
  end;

  // Display the sprites on the screen
  hr := DisplayFrame;
  if (hr <> DD_OK) then
  begin
    if (hr <> DDERR_SURFACELOST) then
    begin
      result := hr;
      exit;
    end;

    // The surfaces were lost so restore them
    RestoreSurfaces;
  end;

  result := DD_OK;
end;

//-----------------------------------------------------------------------------
// Name: AdjustSizeForHardwareLimits()
// Desc: Checks and corrects all boundries for alignment and stretching
//-----------------------------------------------------------------------------
procedure AdjustSizeForHardwareLimits;
begin
  // Setup effects structure
  // Make sure the coordinates fulfill the stretching requirements.  Often
  // the hardware will require a certain ammount of stretching to do
  // overlays. This stretch factor is held in dwMinOverlayStretch as the
  // stretch factor multiplied by 1000 (to keep an accuracy of 3 decimal
  // places).
  if ( (g_ddcaps.dwCaps and DDCAPS_OVERLAYSTRETCH) and
     ( g_ddcaps.dwMinOverlayStretch ) and
     Ord( g_dwXRatio < g_ddcaps.dwMinOverlayStretch ) <> 0 ) then
  begin
    // Window is too small
    g_rcDst.right := 2 * GetSystemMetrics(SM_CXSIZEFRAME) + g_rcDst.left +
                    (SCREEN_WIDTH * (g_ddcaps.dwMinOverlayStretch + 1)) div 1000;
  end;

  if ((g_ddcaps.dwCaps and DDCAPS_OVERLAYSTRETCH) and
     (g_ddcaps.dwMaxOverlayStretch) and
     Ord( g_dwXRatio < g_ddcaps.dwMaxOverlayStretch ) <> 0 ) then
  begin
    // Window is too large
    g_rcDst.right := 2 * GetSystemMetrics(SM_CXSIZEFRAME) + g_rcDst.left +
      (SCREEN_HEIGHT
      * (g_ddcaps.dwMaxOverlayStretch + 999)) div 1000;
  end;

  // Recalculate the ratio's for the upcoming calculations
  g_dwXRatio := (g_rcDst.right - g_rcDst.left) * 1000 div
                (g_rcSrc.right - g_rcSrc.left);
  g_dwYRatio := (g_rcDst.bottom - g_rcDst.top) * 1000 div
                (g_rcSrc.bottom -  g_rcSrc.top);

  // Check to make sure we're within the screen's boundries, if not then fix
  // the problem by adjusting the source rectangle which we draw from.
  if (g_rcDst.left < 0) then
  begin
    g_rcSrc.left := -g_rcDst.left * 1000 div g_dwXRatio;
    g_rcDst.left := 0;
  end;

  if (g_rcDst.right > GetSystemMetrics(SM_CXSCREEN)) then
  begin
    g_rcSrc.right := SCREEN_WIDTH - ((g_rcDst.right -
      GetSystemMetrics(SM_CXSCREEN)) *
      1000 div g_dwXRatio);
    g_rcDst.right := GetSystemMetrics(SM_CXSCREEN);
  end;

  if (g_rcDst.bottom > GetSystemMetrics(SM_CYSCREEN)) then
  begin
    g_rcSrc.bottom := SCREEN_HEIGHT - ((g_rcDst.bottom -
      GetSystemMetrics(SM_CYSCREEN))
      * 1000 div g_dwYRatio);
    g_rcDst.bottom := GetSystemMetrics(SM_CYSCREEN);
  end;

  if (g_rcDst.top < 0) then
  begin
    g_rcSrc.top := -g_rcDst.top * 1000 div g_dwYRatio;
    g_rcDst.top := 0;
  end;

  // Make sure the coordinates fulfill the alignment requirements
  // these expressions (x & -y) just do alignment by dropping low order bits...
  // so to round up, we add first, then truncate.
  if ((g_ddcaps.dwCaps and DDCAPS_ALIGNBOUNDARYSRC) and
    (g_ddcaps.dwAlignBoundarySrc) <> 0 )  then
  begin
    g_rcSrc.left := (g_rcSrc.left + g_ddcaps.dwAlignBoundarySrc div 2) and
                    -(g_ddcaps.dwAlignBoundarySrc);
  end;

  if ((g_ddcaps.dwCaps and DDCAPS_ALIGNSIZESRC) and
    (g_ddcaps.dwAlignSizeSrc) <> 0 )  then
  begin
    g_rcSrc.right := (g_rcSrc.left + (g_rcSrc.right - g_rcSrc.left +
                     g_ddcaps.dwAlignSizeSrc div 2) ) and -(g_ddcaps.dwAlignSizeSrc);
  end;

  if ((g_ddcaps.dwCaps and DDCAPS_ALIGNBOUNDARYDEST) and
    (g_ddcaps.dwAlignBoundaryDest) <> 0 )  then
  begin
    g_rcDst.left := (g_rcDst.left + g_ddcaps.dwAlignBoundaryDest div 2) and
                    -(g_ddcaps.dwAlignBoundaryDest);
  end;

  if ((g_ddcaps.dwCaps and DDCAPS_ALIGNSIZEDEST) and
    (g_ddcaps.dwAlignSizeDest) <> 0 )  then
  begin
    g_rcDst.right := g_rcDst.left + (g_rcDst.right - g_rcDst.left) and
                     -(g_ddcaps.dwAlignSizeDest);
  end;
end;

//-----------------------------------------------------------------------------
// Name: HasOverlaySupport()
// Desc: Returns TRUE if the device supports overlays, FALSE otherwise
//-----------------------------------------------------------------------------

function HasOverlaySupport: Boolean;
begin
  // Get driver capabilities to determine overlay support.
  ZeroMemory(@g_ddcaps, sizeof(g_ddcaps));
  g_ddcaps.dwSize := sizeof(g_ddcaps);
  g_pDD.GetCaps(@g_ddcaps, nil);

  // Does the driver support overlays in the current mode?
  // The DirectDraw emulation layer does not support overlays
  // so overlay related APIs will fail without hardware support.
  if (g_ddcaps.dwCaps and DDCAPS_OVERLAY) <> 0 then
  begin
    // Make sure it supports stretching (scaling)
    if (g_ddcaps.dwCaps and DDCAPS_OVERLAYSTRETCH) <> 0 then
      result := TRUE
    else
      result := FALSE;
  end
  else
  begin
    result := FALSE;
  end;
end;

{-----------------------------------------------------------------------------
   Name: WindowProc
   Desc: The Main Window Procedure
 -----------------------------------------------------------------------------}

function WindowProc(h_Wnd: THandle; aMSG: Cardinal; wParam: Cardinal; lParam:
  Integer): Integer; stdcall;
var
  pDDPal: IDIRECTDRAWPALETTE;
  p: TPOINT;
begin
  case aMSG of
    // Pause if minimized
    WM_COMMAND:
      begin
        case LOWORD(wParam) of
          IDM_EXIT:
            begin
              // Received key/menu command to exit app
              PostMessage(h_Wnd, WM_CLOSE, 0, 0);
              result := 0;
              exit;
            end;
        end;
      end;

    WM_PAINT:
      begin
        // Update the screen if we need to refresh. This case occurs
        // when in windowed mode and the window is behind others.
        // The app will not be active, but it will be visible.
        if (g_pDDSPrimary <> nil) then
        begin
          // UpdateOverlay is how we put the overlay on the screen.
          if (g_rcDst.top = g_rcDst.bottom) then
          begin
            g_pDDSOverlay.UpdateOverlay(nil, g_pDDSPrimary, nil,
              DDOVER_HIDE, nil);
          end
          else
          begin
            g_pDDSOverlay.UpdateOverlay(@g_rcSrc, g_pDDSPrimary,
              @g_rcDst, g_dwOverlayFlags,
              @g_OverlayFX);
          end;
        end;
      end;

    WM_QUERYNEWPALETTE:
      begin
        if (g_pDDSPrimary <> nil) then
        begin
          // If we are in windowed mode with a desktop resolution in 8 bit
          // color, then the palette we created during init has changed
          // since then.  So get the palette back from the primary
          // DirectDraw surface, and set it again so that DirectDraw
          // realises the palette, then release it again.
          pDDPal := nil;
          g_pDDSPrimary.GetPalette(pDDPal);
          g_pDDSPrimary.SetPalette(pDDPal);
          pDDPal := nil;
        end;
      end;

    WM_MOVE:
      begin
        // Make sure we're not moving to be minimized - because otherwise
        // our ratio varialbes (g_dwXRatio and g_dwYRatio) will end up
        // being 0, and once we hit CheckBoundries it divides by 0.
        if (not IsIconic(h_Wnd)) then
        begin
          p.x := 0;
          p.y := 0; // Translation point for the window's client region

          g_rcSrc.left := 0;
          g_rcSrc.right := SCREEN_WIDTH;
          g_rcSrc.top := 0;
          g_rcSrc.bottom := SCREEN_HEIGHT;

          GetClientRect(h_Wnd, g_rcDst);

          g_dwXRatio := (g_rcDst.right - g_rcDst.left) * 1000 div
            (g_rcSrc.right - g_rcSrc.left);
          g_dwYRatio := (g_rcDst.bottom - g_rcDst.top) * 1000 div
            (g_rcSrc.bottom - g_rcSrc.top);

          ClientToScreen(h_Wnd, p);
          g_rcDst.left := p.x;
          g_rcDst.top := p.y;
          g_rcDst.bottom := g_rcDst.bottom + p.y;
          g_rcDst.right := g_rcDst.right + p.x;
          if (g_pDD <> nil) then
            AdjustSizeForHardwareLimits;
        end
        else
        begin
          // Else, hide the overlay... just in case we can't do
          // destination color keying, this will pull the overlay
          // off of the screen for the user.
          if (g_pDDSOverlay <> nil) and (g_pDDSPrimary <> nil) then
            g_pDDSOverlay.UpdateOverlay(nil, g_pDDSPrimary, nil,
              DDOVER_HIDE, nil);
        end;

        // Check to make sure our window exists before we tell it to
        // repaint. This will fail the first time (while the window is
        // being created).
        if (h_Wnd <> 0) then
        begin
          InvalidateRect(h_Wnd, nil, FALSE);
          UpdateWindow(h_Wnd);
        end;

        result := 0;
        exit;
      end;

    WM_SIZE:
      begin
        // Another check for the minimization action.  This check is
        // quicker though...
        // Check to see if we are losing our window...
        if (SIZE_MAXHIDE = wParam) or (SIZE_MINIMIZED = wParam) then
          g_bActive := FALSE
        else
          g_bActive := TRUE;

        if (g_bActive) then
        begin
          p.x := 0;
          p.y := 0; // Translation point for the window's client region

          GetClientRect(h_Wnd, g_rcDst);
          ClientToScreen(h_Wnd, p);

          g_rcDst.left := p.x;
          g_rcDst.top := p.y;
          g_rcDst.bottom := g_rcDst.bottom + p.y;
          g_rcDst.right := g_rcDst.right + p.x;

          g_rcSrc.left := 0;
          g_rcSrc.right := SCREEN_WIDTH;
          g_rcSrc.top := 0;
          g_rcSrc.bottom := SCREEN_HEIGHT;

          // Here we multiply by 1000 to preserve 3 decimal places in the
          // division opperation (we picked 1000 to be on the same order
          // of magnitude as the stretch factor for easier comparisons)
          g_dwXRatio := (g_rcDst.right - g_rcDst.left) * 1000 div
            (g_rcSrc.right - g_rcSrc.left);

          g_dwYRatio := (g_rcDst.bottom - g_rcDst.top) * 1000 div
            (g_rcSrc.bottom - g_rcSrc.top);

          AdjustSizeForHardwareLimits;
        end;
        result := 0;
        exit;
      end;

    WM_DISPLAYCHANGE:
      begin
        // This not only checks for overlay support in the new video mode -
        // but gets the new caps for the new display settings.  That way we
        // have more accurate info about min/max stretch factors, color
        // keying
        if (not HasOverlaySupport) then
        begin
          MessageBox(h_Wnd, 'You have changed your adapter settings such ' +
            ' that you no longer support this overlay.', 'Overlay', MB_OK);
          PostMessage(h_Wnd, WM_CLOSE, 0, 0);
        end;
        result := 0;
        exit;
      end;

    WM_DESTROY:
      begin
        // Cleanup and close the app
        FreeDirectDraw;
        PostQuitMessage(0);
        result := 0;
        exit;
      end;
  end;
  Result := DefWindowProc(h_Wnd, aMSG, wParam, lParam);
end;

{-----------------------------------------------------------------------------
   Name: WinInit
   Desc: Do work required for every instance of the application:
         Create the window, initialize data
-----------------------------------------------------------------------------}

function WinInit(h_Inst: THANDLE; nCmdShow: Integer; var phWnd: THandle; var
  phAccel: HACCEL): HRESULT;
var
  h_Wnd: THandle;
  wc: TWndClassEx;
  h_Accel: HACCEL;
  dwStyle, dwFrameWidth, dwFrameHeight, dwMenuHeight, dwCaptionHeight,
    dwWindowWidth, dwWindowHeight: DWORD;
begin
  // Register the window class
  wc.cbSize := SizeOf(wc);
  wc.lpszClassName := NAME;
  wc.lpfnWndProc := @WindowProc;
  wc.style := CS_VREDRAW or CS_HREDRAW;
  wc.hInstance := h_Inst;
  wc.hIcon := LoadIcon(h_Inst, MakeIntResource(IDI_MAIN_ICON));
  wc.hIconSm := LoadIcon(h_Inst, MakeIntResource(IDI_MAIN_ICON));
  wc.hCursor := LoadCursor(0, IDC_ARROW);

  // Let windows paint the colorkey'ed background which our overlay will use
  wc.hbrBackground := CreateSolidBrush(g_dwBackgroundColor);
  wc.lpszMenuName := MakeIntResource(IDR_MENU);
  wc.cbClsExtra := 0;
  wc.cbWndExtra := 0;

  if (RegisterClassEx(wc) = 0) then
  begin
    result := E_FAIL;
    exit;
  end;

  // Load keyboard accelerators
  h_Accel := LoadAccelerators(h_Inst, MakeIntResource(IDR_MAIN_ACCEL));

  // Calculate the proper size for the window given a client of 640x480
  dwFrameWidth := GetSystemMetrics(SM_CXSIZEFRAME);
  dwFrameHeight := GetSystemMetrics(SM_CYSIZEFRAME);
  dwMenuHeight := GetSystemMetrics(SM_CYMENU);
  dwCaptionHeight := GetSystemMetrics(SM_CYCAPTION);
  dwWindowWidth := SCREEN_WIDTH + dwFrameWidth * 2;
  dwWindowHeight := SCREEN_HEIGHT + dwFrameHeight * 2 + dwMenuHeight +
    dwCaptionHeight;

  // Create and show the main window
  dwStyle := WS_OVERLAPPEDWINDOW;
  h_Wnd := CreateWindowEx(0,
    NAME,
    TITLE,
    dwStyle,
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    dwWindowWidth,
    dwWindowHeight,
    0,
    0,
    h_Inst,
    nil);
  if (h_Wnd = 0) then
  begin
    result := E_FAIL;
    exit;
  end;

  //ShowWindow( h_Wnd, nCmdShow );
  //UpdateWindow( h_Wnd );

  phWnd := h_Wnd;
  phAccel := h_Accel;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: WinMain
   Desc: Entry point to the program. Initializes everything and calls
         UpdateFrame() when idle from the message pump.
-----------------------------------------------------------------------------}
var
  aMSG: MSG;
  h_Accel: HACCEL;
  hr: HRESULT;
begin
  g_rcSrc := RECT(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
  g_rcDst := RECT(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);

  g_dwBackgroundColor := RGB(10, 0, 10);

  hr := WinInit(hInstance, SW_SHOW, h_Wnd, h_Accel);
  if (hr <> DD_OK) then
  begin
    exit;
  end;

  hr := InitDirectDraw(h_Wnd);
  if (hr <> DD_OK) then
  begin
    MessageBox(h_Wnd, 'DirectDraw init failed. ' + #13#10 +
      'The sample will now exit. ', 'DirectDraw Sample', MB_ICONERROR or MB_OK);
    exit;
  end;

  if (not HasOverlaySupport) then
  begin
    MessageBox(h_Wnd, 'This DirectDraw device does not support overlays. ' +
      #13#10 + 'The sample will now exit. ', 'DirectDraw Sample', MB_ICONERROR
      or
      MB_OK);
    exit;
  end;

  // Create the DirectDraw surfaces needed
  hr := CreateDirectDrawSurfaces(h_Wnd);
  if (hr <> DD_OK) then
  begin
    MessageBox(h_Wnd,
      'Failed to create surfaces.  This DirectDraw device may not support overlays. ' + #13#10
      + 'The sample will now exit. ', 'DirectDraw Sample', MB_ICONERROR or
      MB_OK);
    exit;
  end;

  ShowWindow(h_Wnd, SW_SHOW);

  while (true) do
  begin
    // Look for messages, if none are found then
    // update the state and display it
    if (PeekMessage(aMSG, 0, 0, 0, PM_NOREMOVE)) then
    begin
      if not GetMessage(aMSG, 0, 0, 0) then
        // WM_QUIT was posted, so exit
        exit;

      // Translate and dispatch the message
      if (TranslateAccelerator(h_Wnd, h_Accel, aMsg) = 0) then
      begin
        TranslateMessage(aMsg);
        DispatchMessage(aMsg);
      end;
    end
    else if (g_bActive) then
    begin
      // Move the sprites, blt them to the back buffer, then
      // flip or blt the back buffer to the primary buffer
      hr := ProcessNextFrame(h_Wnd);
      if (hr <> DD_OK) then
      begin
        g_pDD.SetCooperativeLevel(0, DDSCL_NORMAL);
        //g_pDisplay := nil;

        MessageBox(h_Wnd, 'Displaying the next frame failed.' + #13#10 +
          'The sample will now exit.', 'DirectDraw Sample', MB_ICONERROR or
          MB_OK);
        exit;
      end;
    end
    else
    begin
      // Go to sleep if we have nothing else to do
      WaitMessage;
    end;
  end;
end.

